import libelf

class Arch:
    def __init__(self, name, **kw):
        self.name = name
        self.page_shift = kw['page_shift']
        self.page_size = 1 << kw['page_shift']
        self.pagemask = (1 << kw['page_shift']) - 1
        self.pt_format = kw['pt_format']
        self.max_vcpu = kw['max_vcpu']
        self.elf_format = kw['elf_format']
        self.elf_machine = kw['elf_machine']

    def maddr_to_mfn(self, maddr):
        '''return mfn from maddr'''
        return maddr >> self.page_shift
        
    def mfn_to_maddr(self, mfn):
        '''return maddr from mfn'''
        return mfn << self.page_shift

    def pfn_to_vaddr(self, pfn):
        '''return vaddr from pfn'''
        return pfn << self.page_shift

    def page_offset(self, maddr):
        '''return offset in page'''
        return maddr & self.pagemask

    def round_pgup(self, size):
        '''return size round by pagesize'''
        return ((size + (self.page_size -1)) / self.page_size) * self.page_size



archparam = {'x86_32':
             {'page_shift': 12,
              'pt_format':'L',
              'max_vcpu': 32,
              'elf_format': 'ELF32',
              'elf_machine': libelf.EM_386,
              },
             'x86_64':
             {'page_shift': 12,
              'pt_format':'Q',
              'max_vcpu': 32,
              'elf_format': 'ELF64',
              'elf_machine': libelf.EM_386,
              },
             'ia64':
             {'page_shift': 14,
              'pt_format':'Q',
              'max_vcpu': 4,
              'elf_format': 'ELF64',
              'elf_machine': libelf.EM_IA_64,
              }}

arch = {'x86_32': Arch('x86_32', **archparam['x86_32']),
        'x86_64': Arch('x86_64', **archparam['x86_64']),
        'ia64': Arch('ia64', **archparam['ia64']),
        }
