import struct
import Register
import StringIO as StringIO

class XenContext:
    def __init__(self, tree, context=None):
        self.tree = tree
        self.ctxt_size = tree.get_struct_size('vcpu_guest_context')
        self.reg_offset = tree.calc_struct_member_offset('vcpu_guest_context', 'user_regs') 
        self.reg_size =  tree.get_struct_size('cpu_user_regs') 
        self.ctrlreg_offset = tree.calc_struct_member_offset('vcpu_guest_context', 'ctrlreg')
        self.ctrlreg_format = 'L'
        self.ctrlreg_size = struct.calcsize('L')
        self.max_ctrlreg = 7            # 0-7

        if context:
            self.context = context
        else:
            self.context = "\x00" * self.ctxt_size

        self.ctxtf = StringIO.StringIO(self.context)

    def __str__(self):
        return self.ctxtf.getvalue()

    def clone(self):
        return XenContext(self.tree, self.ctxtf.getvalue())
        
    def set_register(self, reg):
        regstr = reg.asXen()

        self.ctxtf.seek(self.reg_offset)
        self.ctxtf.write(regstr)

    def get_register(self, reg):
        self.ctxtf.seek(self.reg_offset)
        regstr = self.ctxtf.read(self.reg_size)
        reg = Register.Register()
        reg.fromXen(regstr)
        return reg

    def set_ctrlreg(self, num, val):
        if num > self.max_ctrlreg or num < 0:
            raise IndexError

        regstr = struct.pack(self.ctrlreg_format, val)
        self.ctxtf.seek(self.ctrlreg_offset + self.ctrlreg_size * num)
        self.ctxtf.write(regstr)
        
    def get_ctrlreg(self, num):
        if num > self.max_ctrlreg or num < 0:
            raise IndexError
        
        self.ctxtf.seek(self.ctrlreg_offset + self.ctrlreg_size * num)
        regstr = self.ctxtf.read(self.ctrlreg_size)
        return struct.unpack(self.ctrlreg_format, regstr)

